@@ -8,7 +8,7 @@
 
 use super::Core;
 use crate::{
-    crypto,
+    ed25519,
     peer::PeerUtils,
     routing::{
         command::Command,
@@ -56,7 +56,7 @@ impl Core {
             data_size: RESOURCE_PROOF_DATA_SIZE,
             difficulty: RESOURCE_PROOF_DIFFICULTY,
             nonce,
-            nonce_signature: crypto::sign(&serialized, &self.node.keypair),
+            nonce_signature: ed25519::sign(&serialized, &self.node.keypair),
         }));
 
         self.send_direct_message(
