@@ -8,11 +8,11 @@
 
 use std::mem;
 
-use crate::agreement::Signed;
-use sn_messaging::{node::Proven, SectionAuthorityProvider};
+use crate::dkg::Signed;
+use sn_messaging::{node::SectionSigned, SectionAuthorityProvider};
 use xor_name::Prefix;
 
-type Entry = (Proven<SectionAuthorityProvider>, Signed);
+type Entry = (SectionSigned<SectionAuthorityProvider>, Signed);
 
 // Helper structure to make sure we process a split by updating info about both our section and the
 // sibling section at the same time.
@@ -32,7 +32,7 @@ impl SplitBarrier {
     pub fn process(
         &mut self,
         our_prefix: &Prefix,
-        section_auth: Proven<SectionAuthorityProvider>,
+        section_auth: SectionSigned<SectionAuthorityProvider>,
         key_signed: Signed,
     ) -> Vec<Entry> {
         if !section_auth.value.prefix.is_extension_of(our_prefix) {
